SystemOrganization addCategory: #'Puzzles-GUI'!
SystemOrganization addCategory: #'Puzzles-Core'!
SystemOrganization addCategory: #Puzzles!

Matrix subclass: #IrregularGrid
	instanceVariableNames: 'irregularPuzzleGui'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-Core'!

!IrregularGrid class methodsFor: 'as yet unclassified' stamp: 'mg 7/2/2014 17:06'!
cellClass
	
	^ IrregularGridCell! !

!IrregularGrid class methodsFor: 'instance creation' stamp: 'mg 6/30/2014 14:00'!
newConstrainedPuzzleFromFile: aFileName

	| puzzle |
	puzzle := self newFromFile: aFileName.
	puzzle irregularPuzzleGui openInWorld.
	^ puzzle! !

!IrregularGrid class methodsFor: 'instance creation' stamp: 'mg 7/2/2014 17:09'!
newFromFile: aFileName

	| file charList grid puzzleSize |
	charList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: aFileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:char | charList add: char]].
	puzzleSize := charList size sqrt.
	file close.
	grid := self new: puzzleSize element: 0.
	grid buildGridCellsFrom: charList.
	grid buildGUI.

	^ grid! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:13'!
bottomNeighborOfX: x y: y

	x = self rowCount ifTrue: [^'' asCharacter].
	^ (self at: x+1 at: y) regionId
	! !

!IrregularGrid methodsFor: 'initialize-release' stamp: 'mg 7/2/2014 17:12'!
buildGUI

	| borderLeft borderTop borderBottom borderRight |

	irregularPuzzleGui setMatrixCoordinatesOf: self.

	self withIndicesDo: [:val :row :column | | irregularCell | 
		borderTop := (self topNeighborOfX: row y: column) ~= val regionId.
		borderBottom := (self bottomNeighborOfX: row y: column) ~= val regionId.
		borderLeft := (self leftNeighborOfX: row y: column) ~= val regionId.
		borderRight := (self rightNeighborOfX: row y: column) ~= val regionId.

		irregularCell := irregularPuzzleGui cellAtX: row Y: column.
		borderTop ifTrue: [irregularCell addTopBorder].
		borderBottom ifTrue: [irregularCell addBottomBorder].
		borderLeft ifTrue: [irregularCell addLeftBorder].
		borderRight ifTrue: [irregularCell addRightBorder]].
	! !

!IrregularGrid methodsFor: 'initialize-release' stamp: 'mg 7/2/2014 17:07'!
buildGridCellsFrom: aCharacterList

	"self := Matrix new: self rowCount tabulate: [:i :j | aCharacterList at: (j \\ 10 + (i-1*9))]."
	(1 to: self rowCount) do: [:row |
		(1 to: self columnCount) do: [:column |
			self at: row at: column put: (self class cellClass new regionId: (aCharacterList at: (column \\ 10 + (row-1*9))))]].! !

!IrregularGrid methodsFor: 'initialize-release' stamp: 'mg 7/2/2014 17:15'!
initialize

	super initialize.
	irregularPuzzleGui := IrregularGridGUI new.! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:00'!
irregularPuzzleGui

	^ irregularPuzzleGui! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:13'!
leftNeighborOfX: x y: y

	y = 1 ifTrue: [^'' asCharacter].
	^ (self at: x at: y-1) regionId
	! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:14'!
regionWithIdentifier: aCharacter

	| region |
	region := OrderedCollection new.
	self withIndicesDo: [:cell :row :col | 
		cell regionId = aCharacter ifTrue: [region add: (self at: row at: col)]].
	^ region! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:13'!
rightNeighborOfX: x y: y

	y = self columnCount ifTrue: [^'' asCharacter].
	^ (self at: x at: y+1) regionId
	! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:13'!
topNeighborOfX: x y: y

	x = 1 ifTrue: [^'' asCharacter].
	^ (self at: x-1 at: y) regionId
	! !

IrregularGrid subclass: #StarbattleGrid
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-Core'!

!StarbattleGrid class methodsFor: 'as yet unclassified' stamp: 'mg 7/2/2014 17:06'!
cellClass
	
	^ StarbattleGridCell! !

!StarbattleGrid methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 17:33'!
initialize

	super initialize.
	irregularPuzzleGui := StarbattleGridGUI new.! !

Object subclass: #IrregularGridCell
	instanceVariableNames: 'regionId content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-Core'!

!IrregularGridCell methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:00'!
content

	^ content! !

!IrregularGridCell methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:00'!
content: anObject

	content := anObject! !

!IrregularGridCell methodsFor: 'initialize-release' stamp: 'mg 7/2/2014 17:00'!
initialize

	super initialize.
	regionId := $a.
	content := 0.! !

!IrregularGridCell methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:00'!
regionId

	^ regionId! !

!IrregularGridCell methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:00'!
regionId: anObject

	regionId := anObject! !

!IrregularGridCell methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:03'!
sameRegionAs: anIrregularGridCell

	^ self regionId = anIrregularGridCell regionId! !

IrregularGridCell subclass: #StarbattleGridCell
	instanceVariableNames: 'isStar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-Core'!

!StarbattleGridCell methodsFor: 'initialize-release' stamp: 'mg 7/2/2014 17:01'!
initialize

	super initialize.
	isStar := false.! !

!StarbattleGridCell methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:00'!
isStar

	^ isStar! !

!StarbattleGridCell methodsFor: 'accessing' stamp: 'mg 7/2/2014 17:00'!
isStar: anObject

	isStar := anObject! !

SudokuGUI subclass: #IrregularGridGUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!IrregularGridGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:26'!
createCellsFrom: aMorph
	"This method creates all cells of an irregular grid. Each cell is like the given morph."
	| row grid |
	row := RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		layoutInset: 0;
		cellInset: 0;
		color: Color transparent;
		disableDragNDrop.
	9 timesRepeat: [row addMorph: aMorph copy].

	grid := RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		layoutInset: 0;
		cellInset: 0;
		color: Color gray lighter;
		disableDragNDrop.
	9 timesRepeat: [grid addMorph: row copy].

	^ grid! !

!IrregularGridGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:25'!
initializeGrid 

	grid := self createCellsFrom: IrregularCell new.
	grid borderWidth: 0.! !

!IrregularGridGUI methodsFor: 'updating' stamp: 'mg 6/30/2014 13:53'!
setMatrixCoordinatesOf: anIrregularPuzzleGrid
	"Sets the x and y coordinates of the corresponding matrix."
	grid submorphs withIndexDo:
	[:morph :row | morph submorphs withIndexDo: 
		[:cell :column | 
			cell sudokuModel: anIrregularPuzzleGrid; xCoord: row; yCoord: column.
			sudokuCellDict at: row@column put: cell]].! !

IrregularGridGUI subclass: #StarbattleGridGUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!StarbattleGridGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:30'!
initializeGrid 

	grid := self createCellsFrom: StarbattleCell new.
	grid borderWidth: 0.! !

BorderedMorph subclass: #IrregularCell
	instanceVariableNames: 'sudokuModel xCoord yCoord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:42'!
addBottomBorder

	self borderStyle borderBottom: true.
	self changed.! !

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:43'!
addLeftBorder

	self borderStyle borderLeft: true.
	self changed.! !

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:43'!
addRightBorder

	self borderStyle borderRight: true.
	self changed.! !

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:43'!
addTopBorder

	self borderStyle borderTop: true.
	self changed.! !

!IrregularCell methodsFor: 'event handling' stamp: 'mg 6/30/2014 17:40'!
handlesKeyboard: evt

	^ true! !

!IrregularCell methodsFor: 'initialization' stamp: 'mg 6/30/2014 16:42'!
initialize

	| style |
	super initialize.
	style := IrregularGridBorderStyle new.
	style width: 1.
	style color: Color black.
	self 
		borderStyle: style;
		color: Color white;
		width: 77; height: 77.! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
sudokuModel

	^ sudokuModel! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
sudokuModel: anObject

	sudokuModel := anObject! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
xCoord

	^ xCoord! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
xCoord: anObject

	xCoord := anObject! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
yCoord

	^ yCoord! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
yCoord: anObject

	yCoord := anObject! !

IrregularCell subclass: #StarbattleCell
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!StarbattleCell methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:52'!
addImageMorphForFile: anImageFile

	image := ImageMorph new image: (Form fromFileNamed: anImageFile).
	image image: (image image scaledToSize: self extent).
	self addMorph: image.
	image hide.! !

!StarbattleCell methodsFor: 'event handling' stamp: 'mg 6/30/2014 17:50'!
handlesMouseDown: anEvent

	^ true! !

!StarbattleCell methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:28'!
initialize

	super initialize.
	self
		layoutPolicy: TableLayout new;
		listCentering: #center;
		wrapCentering: #center;
		addImageMorphForFile: 'star.png'.! !

!StarbattleCell methodsFor: 'event handling' stamp: 'mg 6/30/2014 17:53'!
mouseDown: evt

	image visible 
		ifTrue: [image hide]
		ifFalse: [image show].! !

SimpleBorder subclass: #IrregularGridBorderStyle
	instanceVariableNames: 'borderTop borderBottom borderLeft borderRight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderBottom

	^ borderBottom! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderBottom: anObject

	borderBottom := anObject! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderLeft

	^ borderLeft! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderLeft: anObject

	borderLeft := anObject! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderRight

	^ borderRight! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderRight: anObject

	borderRight := anObject! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderTop

	^ borderTop! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderTop: anObject

	borderTop := anObject! !

!IrregularGridBorderStyle methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:48'!
frameRectangle: aRectangle on: aCanvas
	"Borders are drawn with greater borderWidth."
	aCanvas frameAndFillRectangle: aRectangle
		fillColor: Color transparent
		borderWidth: 1
		topLeftColor: Color gray lighter
		bottomRightColor: Color gray.

	self borderTop 
		ifTrue: [aCanvas line: aRectangle topLeft + (2@2) to: aRectangle topRight + (-2@2) width: 4 color: Color black].
	self borderBottom
		ifTrue: [aCanvas line: aRectangle bottomLeft + (2@(-2)) to: aRectangle bottomRight + ((-2)@(-2)) width: 4 color: Color black].
	self borderLeft
		ifTrue: [aCanvas line: aRectangle topLeft + (2@2) to: aRectangle bottomLeft + (2@(-2)) width: 4 color: Color black].
	self borderRight
		ifTrue: [aCanvas line: aRectangle topRight + ((-2)@2) to: aRectangle bottomRight + ((-2)@(-2)) width: 4 color: Color black].! !

!IrregularGridBorderStyle methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 16:35'!
initialize

	super initialize.
	self borderTop: false.
	self borderBottom: false.
	self borderLeft: false.
	self borderRight: false.! !
